require 'active_support'

module ActiveSpy
  # Module that defines methods used to spy on some class methods
  #
  module Spy
    # Default snippet to extends the class with {ActiveSpy::Spy::ClassMethods}
    # when {ActiveSpy::Spy} is included in it.
    #
    def self.included(base)
      base.extend ClassMethods
    end

    # Invokes the callback method on the invoker class. The +callback_type+
    # param tells wether it will be called +:after+ or +before+.
    #
    def invoke_callback(method, callback_type)
      callback_invoker = callback_invoker_class.new(self)
      callback = "#{callback_type}_#{method}"
      return unless callback_invoker.respond_to?(callback)
      callback_invoker.send(callback)
    end

    # Gets the invoker class based on the class' name
    #
    def callback_invoker_class
      ActiveSupport::Inflector.constantize "#{self.class.name}Events"
    end

    # Class methods to be defined in classes that includes {ActiveSpy::Spy}
    #
    module ClassMethods
      # Set watchers for the +method+
      #
      def watch_method(*methods)
        methods.each do |method|
          ActiveSpy::SpyList << { 'class' => name, 'method' => method }
        end
      end
    end
  end
end
